<?php
/* --------------------------------------------------------------
   ProductsApiV2Controller.inc.php 2018-09-10
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2018 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

MainFactory::load_class('HttpApiV2Controller');

/**
 * Class ProductsApiV2Controller
 *
 * Provides a gateway to the ProductWriteService and ProductReadService classes, which handle the shop
 * product resources.
 *
 * @category System
 * @package  ApiV2Controllers
 */
class ProductsApiV2Controller extends HttpApiV2Controller
{
	/**
	 * Product write service.
	 *
	 * @var ProductWriteService
	 */
	protected $productWriteService;
	
	/**
	 * Product read service.
	 *
	 * @var ProductReadService
	 */
	protected $productReadService;
	
	/**
	 * Product JSON serializer.
	 *
	 * @var ProductJsonSerializer
	 */
	protected $productJsonSerializer;
	
	/**
	 * Product list item JSON serializer.
	 *
	 * @var ProductListItemJsonSerializer
	 */
	protected $productListItemJsonSerializer;
	
	/**
	 * Sub resources.
	 *
	 * @var array
	 */
	protected $subresource;
	
	
	/**
	 * Initializes API Controller
	 */
	protected function __initialize()
	{
		$this->productWriteService           = StaticGXCoreLoader::getService('ProductWrite');
		$this->productReadService            = StaticGXCoreLoader::getService('ProductRead');
		$this->productJsonSerializer         = MainFactory::create('ProductJsonSerializer');
		$this->productListItemJsonSerializer = MainFactory::create('ProductListItemJsonSerializer');
		$this->subresource                   = [
			'links'          => 'ProductsLinksApiV2Controller',
			'reviews'        => 'ProductReviewsApiV2Controller',
			'reviews_avg'    => 'ProductReviewsApiV2Controller',
			'product_prices' => 'ProductPricesApiV2Controller'
		];
	}
	
	
	/**
	 * @api             {post} /products Create Product
	 * @apiVersion      2.6.0
	 * @apiName         CreateProduct
	 * @apiGroup        Products
	 *
	 * @apiDescription
	 * Creates a new product record or multiple new product records in the system. To see an example usage take a look
	 * at
	 * `docs/REST/samples/product-service/create_product.php`
	 */
	public function post()
	{
		if($this->uri[1] === 'search')
		{
			return $this->_search();
		}
		
		if($this->_mapResponse($this->subresource))
		{
			return;
		}
		
		$productJsonString = $this->api->request->getBody();
		
		if($this->_isBulkRequest(new StringType($productJsonString))) // Create multiple new Products
		{
			$response = $this->_processBulkInsert(new StringType($productJsonString));
			$this->_linkResponse($response['created']);
		}
		else // Create new Product
		{
			$response = $this->_processSingleInsert(new StringType($productJsonString));
			$this->_linkResponse($response);
		}
		
		$this->_writeResponse($response, $this->_hasErrors($response) ? 400 : 201);
	}
	
	
	/**
	 * @api             {put} /products/:id Update Product
	 * @apiVersion      2.1.0
	 * @apiName         ProductCategory
	 * @apiGroup        Products
	 *
	 * @apiDescription
	 * Use this method to update an existing product record. Take a look in the POST method for more detailed
	 * explanation on every resource property. To see an example usage consider
	 * `docs/REST/samples/product-service/update_product.php`
	 */
	public function put()
	{
		if($this->_mapResponse($this->subresource))
		{
			return;
		}
		
		$productJsonString = $this->api->request->getBody();
		
		if(empty($productJsonString))
		{
			throw new HttpApiV2Exception('Product data were not provided.', 400);
		}
		
		if($this->_isBulkRequest(new StringType($productJsonString)))
		{
			$response = $this->_processBulkUpdate(new StringType($productJsonString));
			$this->_linkResponse($response['affected']);
		}
		elseif(isset($this->uri[1]) && is_numeric($this->uri[1]))
		{
			$response = $this->_processSingleUpdate(new StringType($productJsonString), new IdType($this->uri[1]));
			$this->_linkResponse($response);
		}
		else
		{
			throw new HttpApiV2Exception('Product record ID was not provided or is invalid: ' . gettype($this->uri[1]),
			                             400);
		}
		$this->_writeResponse($response, $this->_hasErrors($response) ? 400 : 200);
	}
	
	
	/**
	 * @api             {delete} /products/:id Delete Product
	 * @apiVersion      2.1.0
	 * @apiName         DeleteProduct
	 * @apiGroup        Products
	 *
	 * @apiDescription
	 * Removes a product record from the database. To see an example usage take a look at
	 * `docs/REST/samples/product-service/remove_product.php`
	 */
	public function delete()
	{
		if($this->_mapResponse($this->subresource))
		{
			return;
		}
		
		// Check if record ID was provided.
		if(!isset($this->uri[1]))
		{
			throw new HttpApiV2Exception('Product record ID was not provided in the resource URL.', 400);
		}
		
		if($this->_isBulkDeleteRequest())
		{
			$response = $this->_processBulkDelete(explode(',', $this->uri[1]));
		}
		elseif(is_numeric($this->uri[1]))
		{
			$response = $this->_processSingleDelete(new IdType($this->uri[1]));
		}
		else
		{
			throw new InvalidArgumentException('Product record ID is invalid.', 400);
		}
		
		$this->_writeResponse($response, $this->_hasErrors($response) ? 400 : 200);
	}
	
	
	/**
	 * @api             {get} /products/:id Get Products
	 * @apiVersion      2.6.0
	 * @apiName         GetProduct
	 * @apiGroup        Products
	 *
	 * @apiDescription
	 * Get multiple or a single product records through a GET request. This method supports all the GET parameters
	 * that are mentioned in the "Introduction" section of this documentation. To see an example usage take a look at
	 * `docs/REST/samples/product-service/remove_product.php`
	 */
	public function get()
	{
		if($this->api->request->get('changed') !== null || $this->api->request->get('modified') !== null
		   || $this->api->request->get('deleted') !== null
		)
		{
			return $this->_changeHistory();
		}
		
		// Handle requests that are a sub resource of the categories controller
		if($this->uri[0] === 'categories')
		{
			$condition = ['search' => ['match' => ['products_to_categories.categories_id' => (int)$this->uri[1]]]];
			if($this->api->request->params('recursive') !== null)
			{
				/* @var CategoryReadService $categoryReadService */
				$categoryReadService = StaticGXCoreLoader::getService('CategoryRead');
				$categories          = $categoryReadService->getCategoryIdsTree(new IdType((int)$this->uri[1]));
				$categories          = $categories->getArray();
				$condition           = ['search' => ['in' => ['products_to_categories.categories_id' => $categories]]];
			}
			
			$this->_search(ProductSearchCondition::createByArray($condition));
			
			return;
		}
		
		if($this->_mapResponse($this->subresource))
		{
			return;
		}
		
		$totalCount = null;
		if($this->uri[1] && is_numeric($this->uri[1])) // Get Single Record
		{
			try
			{
				$products = [$this->productReadService->getProductById(new IdType($this->uri[1]))];
			}
			catch(UnexpectedValueException $e)
			{
				throw new HttpApiV2Exception('Product does not exist.', 404);
			}
		}
		else
		{
			$langParameter = ($this->api->request->get('lang') !== null) ? $this->api->request->get('lang') : 'de';
			
			$languageCode = new LanguageCode(new NonEmptyStringType($langParameter));
			$page         = $this->api->request->get('page') ? new IntType($this->api->request->get('page')) : new IntType(1);
			$limit        = $this->api->request->get('per_page') ? new IntType($this->api->request->get('per_page')) : new IntType(HttpApiV2Controller::DEFAULT_PAGE_ITEMS);
			
			$products   = $this->productReadService->getPagedList($languageCode, $page, $limit)->getArray();
			$totalCount = $this->productReadService->getProductsCount();
		}
		
		$response = [];
		
		foreach($products as $product)
		{
			if($product instanceof ProductInterface)
			{
				$serialized = $this->productJsonSerializer->serialize($product, false);
			}
			else
			{
				$serialized = $this->productListItemJsonSerializer->serialize($product, false);
			}
			
			$response[] = $serialized;
		}
		
		if($this->api->request->get('q') !== null)
		{
			$this->_searchResponse($response, $this->api->request->get('q'));
		}
		
		$this->_sortResponse($response);
		$this->_paginateResponse($response, $totalCount);
		$this->_minimizeResponse($response);
		$this->_linkResponse($response);
		
		// Return single resource to client and not array.
		if(isset($this->uri[1]) && is_numeric($this->uri[1]) && count($response) > 0)
		{
			$response = $response[0];
		}
		
		$this->_writeResponse($response);
	}
	
	
	/**
	 * Sub-Resource Products Search
	 *
	 * This method will search all products with a with an given search condition.
	 *
	 * @see ProductsApiV2Controller::post()
	 */
	protected function _search($searchCondition = null)
	{
		$json = $this->api->request->getBody();
		if($searchCondition === null)
		{
			$searchCondition = ProductSearchCondition::createByJson(new NonEmptyStringType($json));
		}
		
		$page    = $this->api->request->get('page', 1);
		$perPage = $this->api->request->get('per_page', AbstractApiV2Controller::DEFAULT_PAGE_ITEMS);
		$pager   = Pager::create($page, $perPage);
		
		$langParameter = ($this->api->request->get('lang') !== null) ? $this->api->request->get('lang') : 'de';
		$languageCode  = new LanguageCode(new NonEmptyStringType($langParameter));
		
		try
		{
			$products = $this->productReadService->searchProducts($languageCode, $searchCondition, $pager)->getArray();
		}
		catch(Exception $e)
		{
			throw new HttpApiV2Exception($e->getMessage(), 400, $e);
		}
		
		$response = [];
		
		foreach($products as $product)
		{
			$response[] = $this->productListItemJsonSerializer->serialize($product, false);
		}
		
		$this->_sortResponse($response);
		$this->_minimizeResponse($response);
		$this->_linkResponse($response);
		
		$this->_writeResponse($response);
	}
	
	
	/**
	 * History handler for modified, changed and deleted query parameters.
	 *
	 * @throws \HttpApiV2Exception
	 */
	protected function _changeHistory()
	{
		$changed = $this->api->request->get('changed');
		if($changed !== null)
		{
			$modified = $changed;
			$deleted  = $changed;
		}
		else
		{
			$modified = $this->api->request->get('modified');
			$deleted  = $this->api->request->get('deleted');
		}
		
		// Check format of modified and deleted date
		if($modified !== null && !preg_match('/\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2}/', $modified))
		{
			throw new HttpApiV2Exception('Given modified date is invalid. Expected format: 2018-09-25 15:59:01', 400);
		}
		if($deleted !== null && !preg_match('/\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2}/', $deleted))
		{
			throw new HttpApiV2Exception('Given deleted date is invalid. Expected format: 2018-09-25 15:59:01', 400);
		}
		
		$langParameter = ($this->api->request->get('lang') !== null) ? $this->api->request->get('lang') : 'de';
		$languageCode  = new LanguageCode(new NonEmptyStringType($langParameter));
		
		$page    = $this->api->request->get('page', 1) > 0 ? (int)$this->api->request->get('page', 1) : 1;
		$perPage = $this->api->request->get('per_page', 50) > 0 ? (int)$this->api->request->get('per_page', 50) : 50;
		$pager   = Pager::create($page, $perPage);
		
		$response = ['deleted' => [], 'modified' => []];
		
		if($modified !== null)
		{
			$searchCondition = ProductSearchCondition::createByArray(['search' => ['geq' => ['products.products_last_modified' => $modified]]]);
			$orders          = $this->productReadService->searchProducts($languageCode, $searchCondition, $pager);
			foreach($orders as $order)
			{
				$serialized            = $this->productListItemJsonSerializer->serialize($order, false);
				$response['modified'][] = $serialized;
			}
			$this->_linkResponse($response['modified']);
		}
		
		if($deleted !== null)
		{
            $deleteHistoryReadService = DeleteHistoryServiceFactory::readService();
            $dateRange                = DateRange::create(new DateTime($deleted), new DateTime('now'));
            $deletedProducts          = $deleteHistoryReadService->findDeleted($dateRange,
                                                                               DeleteHistoryScope::products());
			/** @var DeleteHistoryReportItem $deletedProduct */
			foreach($deletedProducts as $deletedProduct)
			{
				$response['deleted'][] = [
					'id'   => $deletedProduct->deletedId(),
					'date' => $deletedProduct->deletedAt()->format('Y-m-d H:i:s'),
				];
			}
		}
		
		$this->_writeResponse($response);
	}
	
	
	/**
	 * Checks if the performed request was a bulk request (only for POST and PUT requests).
	 *
	 * @param \StringType $jsonString The complete request body as JSON string
	 *
	 * @return bool
	 */
	protected function _isBulkRequest(StringType $jsonString)
	{
		$json = json_decode($jsonString->asString());
		
		if($json === null && json_last_error() > 0)
		{
			throw new InvalidArgumentException('Provided JSON string is malformed and could not be parsed: '
			                                   . $jsonString->asString());
		}
		
		return is_array($json) && array_keys($json) === range(0, count($json) - 1);
	}
	
	
	/**
	 * Checks if the performed request was a bulk request (only for DELETE requests).
	 *
	 * @return bool
	 */
	protected function _isBulkDeleteRequest()
	{
		return strpos($this->uri[1], ',') !== false;
	}
	
	
	/**
	 * Performs the creation of multiple products from the JSON request body and creates a response array.
	 *
	 * @param \StringType $jsonString The complete request body as JSON string
	 *
	 * @return array The response body
	 */
	protected function _processBulkInsert(StringType $jsonString)
	{
		$products       = $this->productJsonSerializer->decodeJson($jsonString->asString());
		$storedProducts = [];
		$errors         = [];
		
		foreach($products as $product)
		{
			try
			{
				$productId        = $this->productWriteService->createProduct($this->productJsonSerializer->deserialize($product));
				$storedProducts[] = $this->productReadService->getProductById(new IdType($productId));
			}
			catch(Exception $exception)
			{
				$errors[] = $this->_getJsonErrorMessage($exception);
			}
		}
		
		return [
			'created' => $this->productJsonSerializer->serializeProductArray($storedProducts, false),
			'errors'  => $errors
		];
	}
	
	
	/**
	 * Performs the creation of a single product from the JSON request body and creates a response array.
	 *
	 * @param \StringType $jsonString The complete request body as JSON string
	 *
	 * @return array The response body
	 */
	protected function _processSingleInsert(StringType $jsonString)
	{
		try
		{
			if(isset($this->uri[1]) && is_numeric($this->uri[1])) // Duplicate Product
			{
				$productJsonObject = json_decode($jsonString->asString());
				
				if($productJsonObject->categoryId === null || !is_numeric($productJsonObject->categoryId))
				{
					$productJsonObject             = new stdClass;
					$productJsonObject->categoryId = 0; // Default category value.
				}
				
				$productId = $this->productWriteService->duplicateProduct(new IdType($this->uri[1]),
				                                                          new IdType($productJsonObject->categoryId));
			}
			else // Create new Product
			{
				$product   = $this->productJsonSerializer->deserialize($this->productJsonSerializer->decodeJson($jsonString->asString()));
				$productId = $this->productWriteService->createProduct($product);
			}
			
			$this->_locateResource('products', $productId);
			$storedProduct = $this->productReadService->getProductById(new IdType($productId));
			
			return $this->productJsonSerializer->serialize($storedProduct, false);
		}
		catch(Exception $exception)
		{
			return $this->_getJsonErrorMessage($exception);
		}
	}
	
	
	/**
	 * Performs the update of multiple products from the JSON request body and creates a response array.
	 *
	 * @param \StringType $jsonString The complete request body as JSON string
	 *
	 * @return array The response body
	 */
	protected function _processBulkUpdate(StringType $jsonString)
	{
		$products       = $this->productJsonSerializer->decodeJson($jsonString->asString());
		$storedProducts = [];
		$errors         = [];
		
		foreach($products as $product)
		{
			try
			{
				if(!isset($product->id) || !is_numeric($product->id))
				{
					throw new HttpApiV2Exception('Product record ID was not provided or is invalid: '
					                             . gettype($product->id), 400);
				}
				
				$productId     = new IdType($product->id);
				$baseProduct   = $this->productReadService->getProductById($productId);
				$storedProduct = $this->productJsonSerializer->deserialize($product, $baseProduct);
				
				$this->productWriteService->updateProduct($storedProduct);
				$storedProducts[] = $this->productReadService->getProductById($productId);
			}
			catch(Exception $exception)
			{
				$errors[] = $this->_getJsonErrorMessage($exception);
			}
		}
		
		return [
			'affected' => $this->productJsonSerializer->serializeProductArray($storedProducts, false),
			'errors'  => $errors
		];
	}
	
	
	/**
	 * Performs the update of a single product by a given ID from the JSON request body and creates a response array.
	 *
	 * @param \StringType $jsonString The complete request body as JSON string
	 * @param \IdType     $productId  The ID of the updated product
	 *
	 * @return array The response body
	 */
	protected function _processSingleUpdate(StringType $jsonString, IdType $productId)
	{
		// Ensure that the product has the correct product id of the request url
		$productJsonString = $this->_setJsonValue($jsonString->asString(), 'id', $productId->asInt());
		$rawProduct        = $this->productJsonSerializer->decodeJson($productJsonString);
		$product           = $this->productJsonSerializer->deserialize($rawProduct,
		                                                               $this->productReadService->getProductById($productId));
		
		$this->productWriteService->updateProduct($product);
		
		return $this->productJsonSerializer->serialize($this->productReadService->getProductById($productId), false);
	}
	
	
	/**
	 * Performs the removal of multiple products from a list of product IDs and creates a response array.
	 *
	 * @param array $ids The IDs of the products that should be removed
	 *
	 * @return array The response body
	 */
	protected function _processBulkDelete(array $ids)
	{
		$deletedProducts = [];
		$errors          = [];
		foreach($ids as $id)
		{
			try
			{
				$this->_deleteProduct(new IdType($id));
				$deletedProducts[] = $id;
			}
			catch(Exception $exception)
			{
				$errors[] = $this->_getJsonErrorMessage($exception);
			}
		}
		
		return [
			'deleted' => $deletedProducts,
			'errors'  => $errors
		];
	}
	
	
	/**
	 * Performs the removal of a single product by a given product ID and creates a response array.
	 *
	 * @param \IdType $productId The ID of the product that should be deleted
	 *
	 * @return array The response body
	 */
	protected function _processSingleDelete(IdType $productId)
	{
		try
		{
			$this->_deleteProduct(new IdType($this->uri[1]));
		}
		catch(Exception $exception)
		{
			return $this->_getJsonErrorMessage($exception);
		}
		
		// Return response JSON.
		return [
			'code'      => 200,
			'status'    => 'success',
			'action'    => 'delete',
			'resource'  => 'Product',
			'productId' => $productId->asInt()
		];
	}
	
	
	/**
	 * Builds a default response body from a given Exception.
	 *
	 * @param \Exception $exception The Exception that is to be described
	 *
	 * @return array The response body
	 */
	protected function _getJsonErrorMessage(Exception $exception)
	{
		return [
			'errorMessage' => $exception->getMessage(),
			'stacktrace'   => $exception->getTrace()
		];
	}
	
	
	/**
	 * Checks if the response of a bulk request contains any errors.
	 *
	 * @param $response The checked response
	 *
	 * @return bool
	 */
	protected function _hasErrors($response)
	{
		return is_array($response) && array_key_exists('errors', $response) && !empty($response['errors']);
	}
	
	
	/**
	 * Deletes a product by its ID.
	 *
	 * @param \IdType $id The ID of the product that is to be deleted
	 */
	protected function _deleteProduct(IdType $id)
	{
		$this->productWriteService->deleteProductById($id);
	}
}
